﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// <summary>
/// SmartI/O ADC 1~4 테스트
/// SmartX : SmartADC
/// 기능 : 거리,압력,조도,온도,자이로,가속도센서등.. Analog센서값 처리
/// 응용분야 : 센서사용 측정분야
/// 항목 : 거리센서 - SHARP(GP2Y0A41SK0F)
///        압력센서 - Smate(33A-030G-2210)
/// 주의사항 : 
/// </summary>

namespace TechNote52_SmartIO_ADC_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // ReadDetailFitering의 인자값
        private int m_iRawData;
        private int m_iAvailable;

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;

            // SmartDraw BackGround 설정
            smartDraw1.BackLayer.SetPenStyle(Color.Blue, 3);
            smartDraw1.BackLayer.Line(50, 10, 50, 317);
            smartDraw1.BackLayer.Line(10, 297, 500, 297);
            smartDraw1.BackLayer.SetPenStyle(Color.Gray, 1);
            smartDraw1.BackLayer.SetFont(16, Color.White);
            
            // Chart의 가로 그리드 그리기
            int j = 10;
            for (int i = 0; i < 7; i++)
            {
                smartDraw1.BackLayer.Line((70 * i) + 50, 17, (70 * i) + 50, 297);
                smartDraw1.BackLayer.TextOut((70 * i) + 42, 307, j.ToString());
                j += 10;
            }

            // Chart의 세로 그리드 그리기
            j = 455;
            for (int i = 1; i < 10; i++)
            {
                smartDraw1.BackLayer.Line(50, 320 - (i * 30) - 23, 480, 320 - (i * 30) - 23);
                smartDraw1.BackLayer.TextOut(10, 315 - (i * 30) - 26, j.ToString());

                j += 455;
                
                if (i == 8)
                {
                    j = 4095;
                }
            }
            
            // 그래프를 그리기 위한 Chart를 설정
            // (인자를 생략하여 폼디자이너의 속성창에서 설정한 값으로 설정)
            smartDraw1.Chart.SetChartConfig();

            // SmartTimer Interval 설정
            smartTimer1.Interval = 10;
            // PutData() 메서드가 4번 호출되었을 때 Chart화면 갱신
            smartDraw1.Chart.ChartDrawStep = 4;

            // Filtering 설정 영역을 숨김
            smartGroupBox1.Visible = false;
        }

        // Adc 값 읽어서 표시하는 Timer
        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            int iVal = 0, iVal1 = 0, iVal2 = 0, iVal3 = 0;
            double iADCVal = 0, iADCVal1 = 0, iADCVal2 = 0, iADCVal3 = 0;

            if (btnADC0.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                if (chkFiltering.Checked == true)
                {
                    // ADC0 데이터를 얻어온다.
                    iVal = smartADC1.ReadDataDetailFiltering(0, m_iRawData, m_iAvailable);
                }
                else
                {
                    // ADC0 데이터를 얻어온다.
                    iVal = smartADC1.ReadData(0);
                }
                
                // 거리측정계산 함수
                ADC0_DataParsing(iVal);
            }
            else
            {
                labADCValue0.Text = "";
                labADCDistance0.Text = "";
            }

            if (btnADC1.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                if (chkFiltering.Checked == true)
                {
                    // ADC1 데이터를 얻어온다.
                    iVal1 = smartADC1.ReadDataDetailFiltering(1, m_iRawData, m_iAvailable);
                }
                else
                {
                    // ADC1 데이터를 얻어온다.
                    iVal1 = smartADC1.ReadData(1);
                }

                // 압력측정계산 함수
                ADC1_DataParsing(iVal1);
            }
            else
            {
                labADCValue1.Text = "";
                labADCPressure1.Text = "";
            }

            if (btnADC2.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                if (chkFiltering.Checked == true)
                {
                    // ADC2 데이터를 얻어온다.
                    iVal2 = smartADC1.ReadDataDetailFiltering(2, m_iRawData, m_iAvailable);
                }
                else
                {
                    // ADC2 데이터를 얻어온다.
                    iVal2 = smartADC1.ReadData(2);
                }
            }
            else
            {
                labADCValue2.Text = "";
            }

            if (btnADC3.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                if (chkFiltering.Checked == true)
                {
                    // ADC3 데이터를 얻어온다.
                    iVal3 = smartADC1.ReadDataDetailFiltering(3, m_iRawData, m_iAvailable);
                }
                else
                {
                    // ADC3 데이터를 얻어온다.
                    iVal3 = smartADC1.ReadData(3);
                }
            }
            else
            {
                labADCValue3.Text = "";
            }

            // ADC 값 표시
            labADCValue0.Text = iVal.ToString();
            labADCValue1.Text = iVal1.ToString();
            labADCValue2.Text = iVal2.ToString();
            labADCValue3.Text = iVal3.ToString();

            // 그래프에 4채널의 데이터를 표시 하기위한 계산
            iADCVal = ((double)iVal / 4096);
            iADCVal1 = ((double)iVal1 / 4096);
            iADCVal2 = ((double)iVal2 / 4096);
            iADCVal3 = ((double)iVal3 / 4096);

            iVal = (int)((double)270.0 * iADCVal);
            iVal1 = (int)((double)270.0 * iADCVal1);
            iVal2 = (int)((double)270.0 * iADCVal2);
            iVal3 = (int)((double)270.0 * iADCVal3);

            // 그래프의 데이터를 입력 합니다. (인자의 수에 맞추어 그래프의 각각의 계열에 적용됨)            
            smartDraw1.Chart.PutData(iVal, iVal1, iVal2, iVal3);
        }

        private void chkFiltering_Click(object sender, EventArgs e)
        {
            if (chkFiltering.Checked == true)
            {
                smartGroupBox1.Visible = true;
            }
            else
            {
                smartGroupBox1.Visible = false;
            }
        }

        private void btnADCStart_Click(object sender, EventArgs e)
        {
            // DetailFiltering 선택 시
            if (chkFiltering.Checked == true)
            {
                // RawDataCnt값이나 AvailabelCnt 값이 라벨에 입력되어 있지 않다면 실행 취소
                if ((labRawCnt.Text == "") || (labAvailableCnt.Text == ""))
                {
                    SmartX.SmartMessageBox.Show("RawData값과 Available 값을 설정하세요");
                    return;
                }
                else
                {
                    // RawDataCnt 값을 라벨에 입력된 값으로 설정
                    m_iRawData = Convert.ToInt32(labRawCnt.Text);

                    // AvailableCnt값을 라벨에 입력된 값으로 설정
                    m_iAvailable = Convert.ToInt32(labAvailableCnt.Text);

                    // 입력된 AvailableCnt값이 1 ~ RawDataCnt - 2 사이의 값이 아닌 경우 실행 취소
                    if ((m_iAvailable < 1) || (m_iAvailable > (m_iRawData - 2)))
                    {
                        SmartX.SmartMessageBox.Show("Available값 범위는 1 ~ " + (m_iRawData - 2).ToString() + "입니다.");
                        // 값을 초기화
                        labAvailableCnt.Text = "1";
                        return;
                    }
                }
            }

            // ADC Start
            if (btnADCStart.ButtonStatus == SmartX.SmartButton.BUTSTATUS.DOWN)
            {
                smartTimer1.Start();
                chkFiltering.Enabled = false;
            }
            // ADC Stop
            else if (btnADCStart.ButtonStatus == SmartX.SmartButton.BUTSTATUS.UP)
            {
                smartTimer1.Stop();
                chkFiltering.Enabled = true;
            }
        }

        private void SmartLabel_Click(object sender, EventArgs e)
        {
            if (sender == labRawCnt)
            {
                smartKeyboard1.Show(154, 236, labRawCnt);
            }
            else
            {
                smartKeyboard1.Show(336, 236, labAvailableCnt);
            }
        }

        // 센서 특성상 선형적이지 않을경우 구간별로 측정해야 합니다. 
        // ADC0 센서값을 거리로 계산해주는 공식. 20cm ~ 50cm 구간 사이 변환 공식
        // 본 코드는 AD값으로 거리를 계산하는 공식이며 AD값에 따라 거리값의 증/감이 비교적 비례하는 구간을 계산한 공식입니다. 
        // AD값의 구간별로 정확하게 거리를 계산하기위해서는 AD구간별로 계산공식이 필요합니다. 
        private void ADC0_DataParsing(double val)
        {
            // 거리센서의 ADC값 = 1064는 거리20cm, ADC값 = 491은 거리50cm 입니다.
            // ADC값 1064 - 491 = 573(ADC구간) 이고 20cm ~ 50cm(ADC의 거리구간)
            // 계산공식은 573 : 30 = (1064 - val) : x 의 결과값에 20(출발점)을 더합니다.

            // ADC값
            labADCValue0.Text = val.ToString();
            // 거리값
            labADCDistance0.Text = Math.Round(((((1064 - val) * 30) / 573) + 20), 0).ToString() + "cm";
        }

        private void ADC1_DataParsing(double val)
        {
            // Input Valtage
            double dInputValt = (4.5 / 4096.0) * val;

            labADCValue1.Text = val.ToString();

            labADCPressure1.Text = Math.Round(((((dInputValt - 0.5) * 30) / (5 * 0.8))), 0).ToString() + "PSI";
        }
    }
}